﻿using Infragistics.Controls.Charts;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace XamDataChartNoiseDataDemo.ImmediateLineSeries
{
    public class ImmediateLineSeriesWBXView : ImmediateLineSeriesBaseView
    {
        private WriteableBitmap _bitmap;
        private Color _fillColor;
        private Color _strokeColor;

        public ImmediateLineSeriesWBXView(ImmediateLineSeries model)
            : base(model)
        {
        }

        public override void OnRender(DrawingContext drawingContext)
        {
            base.OnRender(drawingContext);

            if (_bitmap == null)
            {
                return;
            }

            drawingContext.DrawImage(_bitmap, Viewport);
        }

        public override void OnRendered()
        {
            base.OnRendered();

            if (_bitmap == null)
            {
                OnResize(Viewport.Width, Viewport.Height);
            }

            using (_bitmap.GetBitmapContext())
            {
                _bitmap.Clear(Colors.Transparent);

                RenderPath(Polygon01);
                RenderPath(Polyline0);
                RenderPath(Polyline1);

                //_bitmap.AddDirtyRect(new System.Windows.Int32Rect(
                //	0, 0,
                //	(int)(_bitmap).Width,
                //	(int)(_bitmap).Height)
                //);
                //_bitmap.Unlock();				
            }

            Model.InvalidateVisual();
        }

        private void RenderPath(Path path)
        {
            //SKPath p = new SKPath();

            PathGeometry pg = path.Data as PathGeometry;
            if (pg == null)
            {
                return;
            }

            _fillColor = Colors.Transparent;
            _strokeColor = Colors.Transparent;

            if (path.Fill != null)
            {
                _fillColor = GetColor(path.Fill, path.Opacity, Model.Opacity);
            }
            if (path.Stroke != null)
            {
                _strokeColor = GetColor(path.Stroke, path.Opacity, Model.Opacity);
            }

            foreach (var figure in pg.Figures)
            {
                RenderFigure(figure);
            }

            //if (path.Fill != null)
            //{
            //	SKPaint fillPaint = GetPaint(path.Fill, path.Opacity, Model.Opacity);
            //	_canvas.DrawPath(p, fillPaint);
            //}
            //if (path.Stroke != null)
            //{
            //	SKPaint strokePaint = GetStrokePaint(path.Stroke, Model.Thickness, path.StrokeThickness, path.Opacity);
            //	_canvas.DrawPath(p, strokePaint);
            //}
        }

        private void RenderFigure(PathFigure figure)
        {
            Point pt = figure.StartPoint;

            var segments = figure.Segments;
            var count = segments.Count;
            for (int i = 0; i < count; i++)
            {
                RenderSegment(figure, segments[i]);
            }
        }

        private void RenderSegment(PathFigure f, PathSegment pathSegment)
        {
            if (!(pathSegment is PolyLineSegment))
            {
                return;
            }


            var poly = (PolyLineSegment)pathSegment;

            int extra = 2;
            if (f.IsClosed)
            {
                extra += 2;
            }

            var points = new int[poly.Points.Count * 2 + extra];

            points[0] = (int)f.StartPoint.X;
            points[1] = (int)f.StartPoint.Y;
            for (var i = 0; i < poly.Points.Count; i++)
            {
                var pt = poly.Points[i];
                points[(i + 1) * 2] = (int)pt.X;
                points[(i + 1) * 2 + 1] = (int)pt.Y;
            }
            if (f.IsClosed)
            {
                points[points.Length - 2] = (int)f.StartPoint.X;
                points[points.Length - 1] = (int)f.StartPoint.Y;
            }

            if (f.IsClosed && _fillColor != Colors.Transparent)
            {
                _bitmap.FillPolygon(points, _fillColor);
            }

            if (OrderedScatterModel.IsAntialiasingEnabled)
            {
                for (var i = 0; i < poly.Points.Count; i++)
                {
                    _bitmap.DrawLineAa(points[(i) * 2],
                    points[(i) * 2 + 1],
                    points[(i + 1) * 2],
                    points[(i + 1) * 2 + 1], _strokeColor);
                }

            }
            else
            {
                _bitmap.DrawPolyline(points, _strokeColor);
            }
        }

        public override void OnResize(double width, double height)
        {
            base.OnResize(width, height);

            //if (_bitmap != null)
            //{
            //    _bitmap = null;
            //}

            var bitmap = new WriteableBitmap((int)width, (int)height, 96, 96, PixelFormats.Pbgra32, null);
            _bitmap = bitmap;
        }

        private Color GetColor(Brush igBrush, double opacity, double globalOpacity)
        {
            Color c = Colors.Red;
            if (igBrush is SolidColorBrush)
            {
                c = ((SolidColorBrush)igBrush).Color;
            }
            else if (igBrush is LinearGradientBrush)
            {
                c = ((LinearGradientBrush)igBrush).GradientStops[0].Color;
            }
            else if (igBrush is RadialGradientBrush)
            {
                c = ((RadialGradientBrush)igBrush).GradientStops[0].Color;
            }

            return c;
        }

        public override void AttachUI(Canvas rootCanvas)
        {
            base.AttachUI(rootCanvas);

            Polygon01.SetBinding(Shape.FillProperty, new Binding("ActualBrush") { Source = Model });
            Polygon01.SetValue(Canvas.ZIndexProperty, 100);
            Polygon01.Opacity = 0.75;

            Polyline0.SetBinding(Shape.StrokeProperty, new Binding("ActualBrush") { Source = Model });
            Polyline0.SetBinding(Shape.StrokeThicknessProperty, new Binding("Thickness") { Source = Model });
            Polyline0.SetBinding(Shape.StrokeDashCapProperty, new Binding("DashCap") { Source = Model });
            Polyline0.SetBinding(Shape.StrokeDashArrayProperty, new Binding("DashArray")
            {
                Source = Model,
                Converter = new DoubleCollectionDuplicator()
            });
            Polyline0.SetBinding(Shape.StrokeLineJoinProperty, new Binding("LineJoin") { Source = Model });
            Polyline0.SetBinding(Shape.StrokeMiterLimitProperty, new Binding("MiterLimit") { Source = Model });
            Polyline0.SetBinding(Shape.StrokeStartLineCapProperty, new Binding("StartCap") { Source = Model });
            Polyline0.SetBinding(Shape.StrokeEndLineCapProperty, new Binding("EndCap") { Source = Model });
            Polyline0.SetBinding(Shape.StrokeLineJoinProperty, new Binding("LineJoin") { Source = Model });
            Polyline0.SetValue(Canvas.ZIndexProperty, 100);

            Polyline1.SetBinding(Shape.StrokeProperty, new Binding("ActualBrush") { Source = Model });
            Polyline1.SetBinding(Shape.StrokeThicknessProperty, new Binding("Thickness") { Source = Model });
            Polyline1.SetBinding(Shape.StrokeDashCapProperty, new Binding("DashCap") { Source = Model });
            Polyline1.SetBinding(Shape.StrokeDashArrayProperty, new Binding("DashArray")
            {
                Source = Model,
                Converter = new DoubleCollectionDuplicator()
            });
            Polyline1.SetBinding(Shape.StrokeLineJoinProperty, new Binding("LineJoin") { Source = Model });
            Polyline1.SetBinding(Shape.StrokeMiterLimitProperty, new Binding("MiterLimit") { Source = Model });
            Polyline1.SetBinding(Shape.StrokeStartLineCapProperty, new Binding("StartCap") { Source = Model });
            Polyline1.SetBinding(Shape.StrokeEndLineCapProperty, new Binding("EndCap") { Source = Model });
            Polyline1.SetBinding(Shape.StrokeLineJoinProperty, new Binding("LineJoin") { Source = Model });
            Polyline1.SetValue(Canvas.ZIndexProperty, 100);


            Model.RenderSeries(false);
        }

        public override void PrepSurface(RenderSurface surface)
        {
            base.PrepSurface(surface);

            // this upscales the size of the thumbnail so that it can more accurately render
            // the data to look more like the main view.  We are, after all, trying to render an entire
            // data set into a tiny space.  This will introduce artifacts and lower visual quality of the thumbnail.
            if (IsThumbnail && OrderedScatterModel.UpscaleOPD)
            {
                Rect windowRect;
                Rect viewportRect;
                MainView.GetViewInfo(out viewportRect, out windowRect);

                Viewport = viewportRect;
            }


            if (Viewport != Rect.Empty)
            {
                OnResize(Viewport.Width, Viewport.Height);
            }

            if (this.RootCanvas != null)
            {
                this.RootCanvas.Children.Clear();
                this.RootCanvas.Children.Add(new Image
                {
                    Source = _bitmap,
                    Stretch = Stretch.Fill,
                    Width = (surface.Surface.Parent as Canvas).ActualWidth,
                    Height = (surface.Surface.Parent as Canvas).ActualHeight,
                });
            }            
        }
    }
}
