﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;

using System.Collections.Specialized;

namespace XamDataChartNoiseDataDemo.ViewModels
{


    public class MultipleSignalViewModel : Models.ObservableObject
    {

        #region Private members


        /// Dock Edge of Legend
        /// </summary>
        Infragistics.Controls.DockEdge _LegendDockEdge = Infragistics.Controls.DockEdge.InsideTop;

        
        /// <summary>
        /// Vertical Dock Alignment of Legend
        /// </summary>
        System.Windows.VerticalAlignment _LegendVerticalDockAlignment = System.Windows.VerticalAlignment.Top;


        /// <summary>
        /// Horizontal Dock Alignment of Legend
        /// </summary>
        System.Windows.HorizontalAlignment _LegendHorizontalDockAlignment = System.Windows.HorizontalAlignment.Right;


        /// <summary>
        /// Collection of all Crankshafts
        /// </summary>
        private ObservableCollection<Models.CrankshaftSignal> _Crankshafts = new ObservableCollection<Models.CrankshaftSignal>();


        /// <summary>
        /// Selected Signals
        /// </summary>
        private ObservableCollection<Models.CrankshaftSignal> _SelectedSignals = null;


        #endregion


        #region Public members


        /// <summary>
        /// Collection of all Crankshafts
        /// </summary>
        public ObservableCollection<Models.CrankshaftSignal> Crankshafts
        {
            get
            {
                return this._Crankshafts;
            }
            set
            {
                SetProperty(ref this._Crankshafts, value);
            }
        }


        /// <summary>
        /// Count
        /// </summary>
        public Int32 CountCrankshafts
        {
            get
            {
                if (this.Crankshafts != null)
                {
                    return this.Crankshafts.Count;
                }
                else
                {
                    return 0;
                }
            }
        }


        public Boolean HasCrankshafts
        {
            get
            {
                return (CountCrankshafts > 0);
            }
        }


        /// <summary>
        /// Selected signals
        /// </summary>
        public ObservableCollection<Models.CrankshaftSignal> SelectedSignals
        {
            get
            {
                return this._SelectedSignals;
            }
            private set
            {
                if (SetProperty(ref this._SelectedSignals, value))
                {
                    OnPropertyChanged(nameof(CountSelectedSignals));
                    OnPropertyChanged(nameof(HasSelectedSignals));
                    OnPropertyChanged(nameof(LegendVisibility));
                }
            }
        }


        /// <summary>
        /// Count
        /// </summary>
        public Int32 CountSelectedSignals
        {
            get
            {
                if (this._SelectedSignals != null)
                {
                    return this._SelectedSignals.Count;
                }
                else
                {
                    return 0;
                }
            }
        }


        public Boolean HasSelectedSignals
        {
            get
            {
                return (CountSelectedSignals > 0);
            }
        }


        /// <summary>
        /// Dock Edge of Legend
        /// </summary>
        public Infragistics.Controls.DockEdge LegendDockEdge
        {
            get
            {
                return this._LegendDockEdge;
            }
            set
            {
                if (SetProperty(ref this._LegendDockEdge, value))
                {

                }
            }
        }


        /// <summary>
        /// Vertical Dock Alignment of Legend
        /// </summary>
        public System.Windows.VerticalAlignment LegendVerticalDockAlignment
        {
            get
            {
                return this._LegendVerticalDockAlignment;
            }
            set
            {
                if (SetProperty(ref this._LegendVerticalDockAlignment, value))
                {

                }
            }
        }


        /// <summary>
        /// Horizontal Dock Alignment of Legend
        /// </summary>
        public System.Windows.HorizontalAlignment LegendHorizontalDockAlignment
        {
            get
            {
                return this._LegendHorizontalDockAlignment;
            }
            set
            {
                if (SetProperty(ref this._LegendHorizontalDockAlignment, value))
                {

                }
            }
        }


        /// <summary>
        /// Legend Visibility
        /// </summary>
        public System.Windows.Visibility LegendVisibility
        {
            get
            {
                if (this.HasSelectedSignals)
                {
                    return System.Windows.Visibility.Visible;
                }
                else
                {
                    return System.Windows.Visibility.Hidden;
                }
            }

        }


        #endregion


        #region Constructor


        public MultipleSignalViewModel(ObservableCollection<Models.CrankshaftSignal> cranks)
        {
            this.Crankshafts = cranks;

            this.SelectedSignals = new ObservableCollection<Models.CrankshaftSignal>();
            this.SelectedSignals.CollectionChanged += SignalsCollectionChanged;
        }


        #endregion



        #region CollectionChanged

        private void SignalsCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            OnPropertyChanged(nameof(SelectedSignals));
            OnPropertyChanged(nameof(CountSelectedSignals));
            OnPropertyChanged(nameof(HasSelectedSignals));
            OnPropertyChanged(nameof(LegendVisibility));
        }

        #endregion

    }


 
}
